--- src/os/unix/ngx_files.c.orig        2009-03-06 16:01:21.000000000 +0300
+++ src/os/unix/ngx_files.c     2009-03-26 23:24:23.000000000 +0300
@@ -7,6 +7,63 @@
 #include <ngx_config.h>
 #include <ngx_core.h>

+int
+rename_mv(char *from, char *to)
+{
+    static char buf[MAXBSIZE];
+    struct stat st;
+    struct timeval time[2];
+    char *bufp;
+    ssize_t wcount;
+    size_t wresid;
+    off_t wtotal;
+    int from_fd, to_fd, rcount;
+
+    if (stat(from, &st) != 0) {
+        //warn("warn: get stat error");
+    } else {
+        from_fd = open(from, O_RDONLY);
+        if (from_fd == -1) {
+            //warn("warn: error from file");
+        } else {
+            to_fd = open(to, O_CREAT|O_EXCL|O_WRONLY, 0644);
+            if (to_fd == -1) {
+                //warn("warn: error to file");
+                close(from_fd);
+                return NGX_ERROR;
+            } else {
+                wtotal = 0;
+                while((rcount = read(from_fd, buf, MAXBSIZE)) > 0) {
+                    for (bufp = buf, wresid = rcount; ;
+                    bufp += wcount, wresid -= wcount) {
+                        wcount = write(to_fd, bufp, wresid);
+                        if (wcount <= 0)
+                            break;
+                        wtotal += wcount;
+                        if (wcount >= (ssize_t)wresid)
+                            break;
+                    }
+                    if (wcount != (ssize_t)wresid)
+                        break;
+                }
+            }
+            close(to_fd);
+        }
+        close(from_fd);
+
+        time[0].tv_sec = st.st_atime;
+        time[0].tv_usec = 0;
+        time[1].tv_sec = st.st_mtime;
+        time[1].tv_usec  = 0;
+        if (utimes(to, time) !=0) {
+            //warn("warn: set utimes error");
+        }
+    }
+    if (unlink(from) != 0) {
+        //warn("warn: error remove file");
+    }
+    return NGX_OK;
+}

 ssize_t
 ngx_read_file(ngx_file_t *file, u_char *buf, size_t size, off_t offset)